@@ -6,7 +6,7 @@
  *
  * GNUTLS is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GNUTLS is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -634,7 +632,8 @@ ksba_asn_tree_dump (ksba_asn_tree_t tree, const char *name, FILE *fp)
 	}
     }
 
-  /* FIXME: release the tree if expanded */
+  if (expand)
+    _ksba_asn_release_nodes (root);
 }
 
 int
@@ -997,12 +996,16 @@ static AsnNode
 copy_tree (AsnNode src_root, AsnNode s)
 {
   AsnNode first=NULL, dprev=NULL, d, down, tmp;
+  AsnNode *link_nextp = NULL;
 
   for (; s; s=s->right )
     {
       down = s->down;
       d = copy_node (s);
-
+      if (link_nextp)
+	*link_nextp = d;
+      link_nextp = &d->link_next;
+      
       if (!first)
         first = d;
       else
@@ -1014,6 +1017,15 @@ copy_tree (AsnNode src_root, AsnNode s)
       if (down)
         {
           tmp = copy_tree (src_root, down);
+	  if (tmp)
+	    {
+	      if (link_nextp)
+		*link_nextp = tmp;
+	      link_nextp = &tmp->link_next;
+	      while (*link_nextp)
+		link_nextp = &(*link_nextp)->link_next;
+	    }
+	      
           if (d->down && tmp)
             { /* Need to merge it with the existing down */
               AsnNode x;
@@ -1063,6 +1075,7 @@ static AsnNode
 do_expand_tree (AsnNode src_root, AsnNode s, int depth)
 {
   AsnNode first=NULL, dprev=NULL, d, down, tmp;
+  AsnNode *link_nextp = NULL;
 
   /* On the very first level we do not follow the right pointer so that
      we can break out a valid subtree. */
@@ -1085,6 +1098,9 @@ do_expand_tree (AsnNode src_root, AsnNode s, int depth)
             }
           down = d->down;
           d = copy_node (d);
+	  if (link_nextp)
+	    *link_nextp = d;
+	  link_nextp = &d->link_next;
           if (s->flags.is_optional)
             d->flags.is_optional = 1;
           if (s->flags.in_choice)
@@ -1105,6 +1121,9 @@ do_expand_tree (AsnNode src_root, AsnNode s, int depth)
               AsnNode x;
 
               x = copy_node (s2);
+	      if (link_nextp)
+		*link_nextp = x;
+	      link_nextp = &x->link_next;
               x->left = *dp? *dp : d;
               *dp = x;
               dp = &(*dp)->right;
@@ -1117,7 +1136,12 @@ do_expand_tree (AsnNode src_root, AsnNode s, int depth)
           d->down = tmp;
         }
       else
-        d = copy_node (s);
+        {
+	  d = copy_node (s);
+	  if (link_nextp)
+	    *link_nextp = d;
+	  link_nextp = &d->link_next;
+	}
 
       if (!first)
         first = d;
@@ -1135,7 +1159,17 @@ do_expand_tree (AsnNode src_root, AsnNode s, int depth)
               tmp = NULL;
             }
           else
-            tmp = do_expand_tree (src_root, down, depth+1);
+            {
+	      tmp = do_expand_tree (src_root, down, depth+1);
+	      if (tmp)
+		{
+		  if (link_nextp)
+		    *link_nextp = tmp;
+		  link_nextp = &tmp->link_next;
+		  while (*link_nextp)
+		    link_nextp = &(*link_nextp)->link_next;
+		}
+	    }
           if (d->down && tmp)
             { /* Need to merge it with the existing down */
               AsnNode x;
@@ -1153,6 +1187,7 @@ do_expand_tree (AsnNode src_root, AsnNode s, int depth)
             }
         }
     }
+
   return first;
 }
 
@@ -1179,6 +1214,7 @@ AsnNode
 _ksba_asn_insert_copy (AsnNode node)
 {
   AsnNode n;
+  AsnNode *link_nextp;
 
   n = copy_tree (node, node);
   if (!n)
@@ -1186,7 +1222,13 @@ _ksba_asn_insert_copy (AsnNode node)
   return_null_if_fail (n->right == node->right);
   node->right = n;
   n->left = node;
-  
+
+  /* FIXME: Consider tail pointer for faster insertion.  */
+  link_nextp = &node->link_next;
+  while (*link_nextp)
+    link_nextp = &(*link_nextp)->link_next;
+  *link_nextp = n;
+
   return n;
 }
 
