@@ -766,7 +766,7 @@ static gpg_error_t
 decoder_next (BerDecoder d)
 {
   struct tag_info ti;
-  AsnNode node;
+  AsnNode node = NULL;
   gpg_error_t err;
   DECODER_STATE ds = d->ds;
   int debug = d->debug;
@@ -1020,7 +1020,7 @@ _ksba_ber_decoder_dump (BerDecoder d, FILE *fp)
   if (!d)
     return gpg_error (GPG_ERR_INV_VALUE);
 
-  d->debug = !!getenv("DEBUG_BER_DECODER");
+  d->debug = !!getenv("KSBA_DEBUG_BER_DECODER");
   d->use_image = 0;
   d->image.buf = NULL;
   err = decoder_init (d, NULL);
@@ -1122,7 +1122,7 @@ _ksba_ber_decoder_decode (BerDecoder d, const char *start_name,
   if (r_root)
     *r_root = NULL;
 
-  d->debug = !!getenv("DEBUG_BER_DECODER");
+  d->debug = !!getenv("KSBA_DEBUG_BER_DECODER");
   d->honor_module_end = 1;
   d->use_image = 1;
   d->image.buf = NULL;
