@@ -5,7 +5,7 @@
  *
  * KSBA is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * KSBA is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -47,6 +45,7 @@
 
 #define xfree(a)  ksba_free (a)
 
+static int verbose;
 static int errorcount = 0;
 
 
@@ -97,6 +96,7 @@ print_sexp (ksba_const_sexp_t p)
           else if (*p == ')')
             {
               putchar (*p);
+              p++;
               if (--level <= 0 )
                 return;
             }
@@ -108,7 +108,8 @@ print_sexp (ksba_const_sexp_t p)
           else
             {
               char *endp;
-              unsigned long n;
+              unsigned long n, i;
+              int need_hex;
 
               n = strtoul (p, &endp, 10);
               p = endp;
@@ -117,10 +118,33 @@ print_sexp (ksba_const_sexp_t p)
                   fputs ("[invalid s-exp]", stdout);
                   return;
                 }
-              putchar('#');
-              for (p++; n; n--, p++)
-                printf ("%02X", *p);
-              putchar('#');
+              p++;
+              for (i=0; i < n; i++)
+                if ( !((p[i] >='A' && p[i] <= 'Z')
+                       || (p[i] >='a' && p[i] <='z')
+                       || (p[i] >='0' && p[i] <='9')
+                       || p[i] == '-'
+                       || p[i] == '.'))
+                  break;
+              need_hex = (i<n);
+              if (!n /* n==0 is not allowed, but anyway.  */
+                  || (!need_hex 
+                      && !((*p >='A' && *p <= 'Z') || (*p >='a' && *p <='z'))))
+                printf ("%lu:", n);
+
+              if (need_hex)
+                {
+                  putchar('#');
+                  for (; n; n--, p++)
+                    printf ("%02X", *p);
+                  putchar('#');
+                }
+              else
+                {
+                  for (; n; n--, p++)
+                    putchar (*p);
+                }
+              putchar(' ');
             }
         }
     }
@@ -557,6 +581,13 @@ one_file (const char *fname)
         unsigned char *der;
         size_t derlen;
 
+        if (verbose)
+          {
+            fputs ("  pubkey....: ", stdout);
+            print_sexp (public);
+            putchar ('\n');
+          }
+
         err = _ksba_keyinfo_from_sexp (public, &der, &derlen);
         if (err)
           {
@@ -604,15 +635,16 @@ one_file (const char *fname)
       }
   }
 
-  list_extensions (cert);
+  if (verbose)
+    {
+      sexp = ksba_cert_get_sig_val (cert);
+      fputs ("  sigval....: ", stdout);
+      print_sexp (sexp);
+      ksba_free (sexp);
+      putchar ('\n');
+    }
 
-#if 0
-  sexp = ksba_cert_get_sig_val (cert);
-  fputs ("  sigval....: ", stdout);
-  print_sexp (sexp);
-  ksba_free (sexp);
-  putchar ('\n');
-#endif
+  list_extensions (cert);
 
   ksba_cert_release (cert);
   err = ksba_cert_new (&cert);
@@ -644,9 +676,21 @@ main (int argc, char **argv)
   if (!srcdir)
     srcdir = ".";
 
-  if (argc > 1)
+  if (argc)
     {
-      for (argc--, argv++; argc; argc--, argv++)
+      argc--; argv++;
+    }
+
+  if (argc && !strcmp (*argv, "--verbose"))
+    {
+      verbose = 1;
+      argc--; argv++;
+    }
+
+
+  if (argc)
+    {
+      for (; argc; argc--, argv++)
         one_file (*argv);
     }
   else
