@@ -5,7 +5,7 @@
  *
  * KSBA is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * KSBA is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -188,7 +186,14 @@ one_file (const char *fname)
           printf ("signer %d - digest algo: %s\n", idx, algoid?algoid:"?");
 
           dn = ksba_cms_get_sig_val (cms, idx);
-          printf ("signer %d - signature %s\n", idx, dn? "found": "missing");
+          if (dn)
+            {
+              printf ("signer %d - signature: ", idx);
+              print_sexp (dn);
+              putchar ('\n');
+            }
+          else
+            printf ("signer %d - signature not found\n", idx);
           ksba_free (dn);
         }
     }
